/************************************************************************
 *                              functions.c                             *
 *                                                                      *
 *  2005-05-28 03:01 EDT                                                *
 *  Copyright  2005  Otavio Salvador <otavio@debian.org>                *
 *                   Luis Mondesi    <lemsx1@gmail.com>                 *
 ************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

/*
 * Standard headers 
 */
#ifndef _GNU_SOURCE
/*
 * Allow GNU specific funtions: *_unlocked() in stdio.h 
 */
#define _GNU_SOURCE
#endif /* _GNU_SOURCE */
#include <errno.h>              /* needed by all file manipulating functions */
#include <stdio.h>              /* NULL and others */
#include <stdlib.h>             /* strtol() exit() */
#include <sys/types.h>          /* needed by regex.h regcomp() */
#include <sys/stat.h>           /* stat() */
#include <unistd.h>             /* fork() getpid() unlink() */
#include <pthread.h>            /* pthread functions and data structures */
/*
 * TODO get rid of string.h. See FIXME's 
 */
#include <string.h>             /* strlen() and str*() fam */
#include <signal.h>             /* kill() */
#include <time.h>               /* nanosleep() */
#include <regex.h>              /* regexec() regcomp() */
#include <glib.h>               /* too many... */
#include <glib/gstdio.h>        /* g_fopen() */
#include <fcntl.h>              /* open() */
#include <linux/vt.h>           /* VT_* */
#include <sys/ioctl.h>          /* ioctl() */

#include "common_macros.h"
#include "xml_parser.h"
#include "xml_config.h"
#include "functions.h"
#include "video.h"

static gint arg_progress = 0;
static guint _video_lock = 0;   /* stop splashy from exiting if progressbar
                                 * is being updated */
static splashy_video_t video;
static DFBRectangle progressbar;

/**
 * @desc allows switching virtual terminals
 */
        void
chvt (gint vt_no)
{
        gint fd = open ("/dev/tty0", O_RDWR);
        if (fd < 0)
                return;
        if (!ioctl (fd, VT_ACTIVATE, vt_no))
                ioctl (fd, VT_WAITACTIVE, vt_no);
        close (fd);
}

/**
 * @desc takes care of initilizing all special files needed
 * by splashy
 */
        gboolean
spl_init ()
{
        DEBUG_PRINT ("Entering spl_init(%s)", "");
        if (!init_config (SPL_CONFIG_FILE))
                return FALSE;
        const gchar *_spl_fifo_path = xml_parser_get_text (SPL_FIFO_XPATH);
        struct stat _stat;
        /*
         * re-Create the splash FIFO 
         */
        /*
         * FIXME reusing same fifo file if found unlink(_spl_fifo_path); 
         */
        stat (_spl_fifo_path, &_stat);
        if (!S_ISFIFO (_stat.st_mode))
        {
                DEBUG_PRINT ("spl_init() attempting to create fifo %s",
                                _spl_fifo_path);
                if (mkfifo (_spl_fifo_path, 0700) == 0)
                {
                        return TRUE;
                }
                else
                {
                        ERROR_PRINT ("fatal: FIFO creation failed %s",
                                        _spl_fifo_path);
                        return FALSE;   /* assumes that it will fail */
                }
        }
        DEBUG_PRINT ("reusing FIFO %s", _spl_fifo_path);
        return TRUE;            /* assumes file exists */
}

        gint
cmd_allow_chvt (void **args)
{
        DEBUG_PRINT ("%s\n", "cmd_allow_chvt() called");
        video_allow_vt_switching ();
        return 0;
}

        gint
cmd_exit (void **args)
{
        /*
         * TODO: getting splashy to switch consoles (tty's) automatically
         * after finishing booting turned out to be a bad idea. I'm living
         * the code here commented out just in case we find a solution for
         * this issue later. For now, we wait until we exit, and then send a
         * chvt from splashy99 when splashy has already exited. Users should
         * customize the tty to which splashy will be switched from
         * /etc/default/splashy (on debian). Nonetheless, we let
         * video_allow_vt_switching() here just don't set vt-switching
         * directfb option there as this breaks splashy. - Luis Mondesi
         * <lemsx1@gmail.com> 
         */
        DEBUG_PRINT ("cmd_exit called %s", "");
        video_allow_vt_switching ();

        /*
         * if we are allowing progressbars, then we need to wait until the
         * thread that reads the fifo and updates the progressbar finishes
         * before we send an exit signal 
         */
        const gchar *show_progress =
                xml_parser_get_text ("/splashy/progressbaronboot");
        gboolean progress =
                (g_ascii_strncasecmp (show_progress, "yes", 3) ==
                 0) ? TRUE : FALSE;
        struct timespec _sleep;
        _sleep.tv_sec = 0;
        _sleep.tv_nsec = 10;

        if (progress == TRUE)
        {
                while (_video_lock)
                {
                        nanosleep (&_sleep, NULL);
                }
        }

        /*
         * TODO we should do a nice alpha routine here Fade-to-black comes to 
         * mind 
         */

        /*
         * free up memory 
         */
        video.ev_buffer->Release (video.ev_buffer);     /* input buffer */
        video.keyboard->Release (video.keyboard);
        video.surface->Release (video.surface); /* pix holder */
        video.dfb->Release (video.dfb); /* kill it ! */

        const gchar *_pid_file = xml_parser_get_text ("/splashy/pid");
        if (g_file_test (_pid_file, G_FILE_TEST_EXISTS))
        {
                unlink (_pid_file);     /* remove your own pid before exiting 
                                         */
        }

        exit (0);
        return 0;               /* we never reach this */
}

        gint
cmd_progress (void **args)
{
        if (*(gint *) args[0] < 0 || *(gint *) args[0] > PROGRESS_MAX)
                return -1;

        arg_progress = *(gint *) args[0];

        if (arg_progress == 0)
                return 1;

        /*
         * allow time for display to be drawn 
         */
        _video_lock = 1;
        video_update_progressbar (&video, &progressbar, arg_progress);
        _video_lock = 0;
        return 0;
}

        gint
cmd_chvt (void **args)
{
        /*
         * vt[1-7] 
         */
        if (*(gint *) args[0] < 1 || *(gint *) args[0] > 7)
                return -1;
        video_allow_vt_switching ();    /* tell libdirectfb that's ok to
                                         * allow vt switching */
        chvt (*(gint *) args[0]);
        return 0;
}

/**
 * @desc our command-holding object and a list of supported commands
 *
 * TODO implement the rest of the commands
 */
struct
{
        const gchar *cmd;
        gint (*handler) (void **);
        gint args;
        gchar *specs;
} known_cmds[] =
{
        /*
         * { .cmd = "set theme", .handler = cmd_set_theme, .args = 1, .specs
         * = "s" },
         * 
         * { .cmd = "set mode", .handler = cmd_set_mode, .args = 1, .specs =
         * "s" },
         * 
         * { .cmd = "set message", .handler = cmd_set_mesg, .args = 1, .specs 
         * = "s" },
         * 
         * { .cmd = "repaint", .handler = cmd_repaint, .args = 0, .specs =
         * NULL, },
         */
        {
                .cmd = "progress",.handler = cmd_progress,.args = 1,.specs =
                        "d"},
        {
                .cmd = "chvt",.handler = cmd_chvt,.args = 1,.specs = "d",},
        {
                .cmd = "allowchvt",.handler = cmd_allow_chvt,.args =
                        0,.specs = NULL,},
        {
                .cmd = "exit",.handler = cmd_exit,.args = 0,.specs = NULL,},};

        inline gboolean
spl_test_env_pattern (const gchar * perr_pattern, const gchar * filename)
{
        regex_t preg;
        gint max_buf = 1024;    /* we need at least this number of characters 
                                 * to check */
        gboolean ret = FALSE;   /* assumes we will fail */
        if (!g_file_test (filename, G_FILE_TEST_EXISTS))
        {
                ERROR_PRINT ("Cannot test file %s\n", filename);
                return FALSE;
        }
        FILE *fp;
        gchar *buffer = g_try_malloc (max_buf);
        if (buffer && (fp = g_fopen (filename, "r")))
        {
                fread_unlocked (buffer, max_buf, 1, fp);        /* *_unlocked 
                                                                 * are a GNU
                                                                 * extension: 
                                                                 * makes
                                                                 * function
                                                                 * fread()
                                                                 * thread
                                                                 * safe */
                fclose (fp);
                buffer[max_buf - 1] = '\0';
                if (regcomp (&preg, perr_pattern,
                                        REG_EXTENDED | REG_ICASE | REG_NEWLINE |
                                        REG_NOSUB) == 0)
                {
                        if (regexec (&preg, buffer, 0, NULL, 0) == 0)
                        {
                                ret = TRUE;
                        }
                        else
                        {
                                g_printerr
                                        ("WARN: %s was not found in %s. Did you append vga=0x317 to your kernel parameters? See README for more\n",
                                         perr_pattern, filename);
                        }
                        regfree (&preg);
                }
        }
        if (buffer)
                g_free (buffer);
        return ret;
}

/**
 * @desc reads file for pattern for known failure text messages
 * @param perr_pattern a valid regular expression pattern. @see man regex
 * @param filename a valid filename. Can be a special filename like "/dev/vcs1"
 * @return true if error was found
 */
        inline gboolean
spl_search_pattern (const gchar * perr_pattern, const gchar * filename)
{
        regex_t preg;
        gint max_buf = 6200;    /* vcs is 6144 chars long. no new-lines are
                                 * added. man vcs */
        gboolean ret = FALSE;   /* assumes we will fail */
        if (!g_file_test (filename, G_FILE_TEST_EXISTS))
        {
                ERROR_PRINT ("Cannot test file %s\n", filename);
                return FALSE;
        }
        FILE *fp;
        gchar *buffer = g_try_malloc (max_buf);
        if (buffer && (fp = g_fopen (filename, "r")))
        {
                fread_unlocked (buffer, max_buf, 1, fp);        /* *_unlocked 
                                                                 * are a GNU
                                                                 * extension: 
                                                                 * makes
                                                                 * function
                                                                 * fread()
                                                                 * thread
                                                                 * safe */
                fclose (fp);
                buffer[max_buf - 1] = '\0';
                if (regcomp (&preg, perr_pattern,
                                        REG_EXTENDED | REG_ICASE | REG_NEWLINE |
                                        REG_NOSUB) == 0)
                {
                        if (regexec (&preg, buffer, 0, NULL, 0) == 0)
                                ret = TRUE;
                        regfree (&preg);
                }
        }
        if (buffer)
                g_free (buffer);
        return ret;
}

/**
 * @desc a simple function that waits for text to be written to a named pipe and dispatches commands accordingly
 * @param _fifo named pipe to read
 * @return
 */
        inline void
spl_read_fifo (FILE * _fifo)
{
        DEBUG_PRINT ("reading fifo %s...\n", "");
        gchar buf[1024]; /** used as buffer for reading FIFO */
        gint i, j, k;

        /*
         * Read FIFO and call functor to handle exceptions 
         */
        while (fgets (buf, 1024, _fifo))
        {
                gchar *t;
                int args_i[4];
                void *args[4];

                buf[1023] = 0;
                /*
                 * FIXME buf[g_utf8_strlen(buf,1024)-1] = 0; 
                 */
                buf[strlen (buf) - 1] = 0;

                for (i = 0; i < sizeof (known_cmds) / sizeof (known_cmds[0]);
                                i++)
                {
                        k = strlen (known_cmds[i].cmd);
                        /*
                         * FIXME k = g_utf8_strlen(known_cmds[i].cmd,1024); 
                         */

                        /*
                         * FIXME if (g_ascii_strncasecmp(buf,
                         * known_cmds[i].cmd, k)) 
                         */
                        if (strncmp (buf, known_cmds[i].cmd, k))
                                continue;

                        for (j = 0; j < known_cmds[i].args; j++)
                        {

                                for (; buf[k] == ' '; buf[k] = 0, k++);
                                if (!buf[k])
                                        goto out;

                                switch (known_cmds[i].specs[j])
                                {

                                        case 's':
                                                args[j] = &(buf[k]);
                                                for (; buf[k] != ' '; k++);
                                                break;

                                        case 'd':
                                                /*
                                                 * FIXME args_i[j] =
                                                 * g_ascii_strtod(&(buf[k]), &t); 
                                                 */
                                                args_i[j] =
                                                        strtol (&(buf[k]), &t, 10);
                                                if (t == &(buf[k]))
                                                        goto out;

                                                args[j] = &(args_i[j]);
                                                k = t - buf;
                                                break;
                                }
                        }
                        /**
                         * cmds should be used for everything
                         * @see known_cmds
                         * and handlers
                         */
                        known_cmds[i].handler (args);
                }
        }
out:fclose (_fifo);
}

/**
 * @desc a helper function to construct a filename path from a given xpath
 * It reads config.xml for a given theme and builds the path accordingly
 * @param theme_xpath use this theme
 * @param image_xpath path representing string to retrieve from xml file
 * @return full path to image
 */
        static inline const gchar *
_splashy_theme_image_path (const gchar * theme_xpath,
                const gchar * image_xpath)
{
        DEBUG_PRINT ("_splashy_theme_image_path(theme_xpath,%s) called",
                        image_xpath);
        GString *_ret_path = g_string_new ("");

        const gchar *_path = xml_parser_get_text (image_xpath);

        /*
         * is this a relative path? 
         */
        if (g_ascii_strncasecmp (_path, "/", 1) != 0)
        {
                /*
                 * root dir to themes: i.e. /etc/splashy/themes 
                 */
                g_string_append (_ret_path,
                                xml_parser_get_text (SPL_THEMES_DIR));
                g_string_append (_ret_path, "/");
                /*
                 * current theme name 
                 */
                g_string_append (_ret_path,
                                xml_parser_get_text (theme_xpath));
                g_string_append (_ret_path, "/");
                if (!g_file_test (_ret_path->str, G_FILE_TEST_IS_DIR))
                {
                        ERROR_PRINT ("Directory '%s' does not exist",
                                        _ret_path->str);
                        return NULL;
                }
                /*
                 * image name 
                 */
                g_string_append (_ret_path, _path);
        }
        else
        {
                /*
                 * assume this is the full path to the image 
                 */
                g_string_append (_ret_path, _path);
        }
        return _ret_path->str;
}
        inline void *
fifo_loop (void *data)
{
        /*
         * It's important that the progress bar code runs as a child process
         * because this doesn't need to grab events from the keyboard. All
         * communication to this is done through a FIFO 
         */

        const gchar *_spl_fifo_path = xml_parser_get_text (SPL_FIFO_XPATH);

        /*
         * 0 seconds and 10 nano seconds: 10 x 1x10^-9 
         */
        struct timespec _sleep;
        _sleep.tv_sec = 0;
        _sleep.tv_nsec = 10;

        guint i = 0;
        FILE *spl_fifo = NULL;
        gboolean _preview = *(gboolean *) data;

        /*
         * run our progress bar in a tight loop 
         */
        while (1)
        {
                /*
                 * if we are in test mode, update progressbar continuesly 
                 */
                if (_preview == TRUE)
                {
                        i = (i >= 100) ? 0 : (i + 10);
                        DEBUG_PRINT ("test mode: updating progress bar by %d",i);
                        video_update_progressbar (&video, &progressbar, i);
                        /*
                         * please pause for a 1.10 seconds... 
                         */
                        _sleep.tv_sec = 1;
                        nanosleep (&_sleep, NULL);
                }
                else
                {
                        /*
                         * if no special keys were pressed, read from our
                         * fifo and continue waiting for events 
                         */
                        spl_fifo = g_fopen (_spl_fifo_path, "r");
                        if (!spl_fifo)
                        {
                                if (errno == EINTR)
                                {
                                        /*
                                         * no events in FIFO yet? release
                                         * some CPU cycles 
                                         */
                                        nanosleep (&_sleep, NULL);
                                        continue;
                                }
                                g_print ("Can't open the splash FIFO %s for reading", _spl_fifo_path);
                                pthread_exit (NULL);
                        }
                        /*
                         * fifo is read and a 'handler' takes care of the
                         * rest 
                         */
                        spl_read_fifo (spl_fifo);
                }
        }                       /* ends while(1) */
        pthread_exit (NULL);    /* never reached */
}
        inline void *
keyevent_loop (void *data)
{
        /*
         * setup an event interface 
         */
        DFBInputEvent event;
        /*
         * get autoverboseonerror from config file and set autoverbose
         * accordingly 
         */
        gboolean autoverbose = FALSE;
        if (g_ascii_strncasecmp
                        (xml_parser_get_text (SPL_AUTO_VERBOSE_ON_ERROR), "yes", 3) == 0)
                autoverbose = TRUE;
        while (1)
        {
                /*
                 * sub-parent (we are a fork after all). init is our parent 
                 */
                DFBCHECK (video.ev_buffer->WaitForEvent (video.ev_buffer));
                /*
                 * get all events from our event buffer (fifo style) and
                 * process them. 
                 */
                while (video.ev_buffer->HasEvent (video.ev_buffer) == DFB_OK
                                && video.ev_buffer->GetEvent (video.ev_buffer,
                                        DFB_EVENT (&event)) ==
                                DFB_OK)
                {
                        /*
                         * exit if ESC or F2 is pressed also kill our
                         * sub_child while at it 
                         */
                        if (event.type == DIET_KEYPRESS)
                        {
                                if (event.key_id == DIKI_ESCAPE)
                                {
                                        splashy_child_exit ();
                                }
                                else if (event.key_id == DIKI_F2)
                                {
                                        /*
                                         * FIXME we need to go into VERBOSE
                                         * and show the console in a
                                         * mini-text console inside splashy 
                                         */
                                        splashy_child_exit ();
                                        /* FIXME video_verbose_text(&video); */
                                }
                        }
                }               /* ends while ev_buffer */
                /*
                 * now read the /dev/vcs1 console and look for error patterns 
                 */
                if (autoverbose == TRUE)
                {
                        DEBUG_PRINT
                                ("autoverbose: searching /dev/vcs1 for '%s'",
                                 "(fail|error|abort)");
                        /*
                         * grouping is needed 
                         */
                        if (spl_search_pattern
                                        ("(fail|error|abort)", "/dev/vcs1") == TRUE)
                        {
                                /*
                                 * TODO should we switch to splashy errorimg
                                 * instead of completely exiting?? 
                                 */
                                splashy_child_exit ();
                        }
                }
                else
                {
                        DEBUG_PRINT ("autoverboseonerror is not set to '%s'?",
                                        "yes");
                }
        }                       /* end while(1) */
        pthread_exit (NULL);    /* never reached */
}

/**
 * @desc _splashy_child handles both start and stop routines for splashy during boot and shutdown respectively
 * Note that the fork() happens before this function is called by its
 * wrappers: spl_child_start() and spl_child_stop()
 * @see spl_child_start() and @see spl_child_stop()
 * @param seq sequence command to execute: "boot" or "shutdown"
 * @param show_progress whether we should display a progress bar
 * @return
 */
        gint
_splashy_child (const gchar * seq, gboolean show_progress)
{
        gint thr_id_c, thr_id_d;        /* thread ID for the newly created
                                         * thread */
        pthread_t p_thread_c, p_thread_d;       /* we need thread C and D to
                                                 * manage our keyboard events 
                                                 * * * * and FIFO */
        const gchar *background = NULL;
        gboolean _preview = FALSE;

        if (g_ascii_strncasecmp (seq, "boot", 4) == 0)
        {
                background =
                        _splashy_theme_image_path (SPL_CURRENT_THEME,
                                        "/splashy/background/boot");
                /*
                 * fallback to default theme if the current theme has no
                 * image defined 
                 */
                if (!g_file_test (background, G_FILE_TEST_IS_REGULAR))
                        background =
                                _splashy_theme_image_path (SPL_DEFAULT_THEME,
                                                "/splashy/background/boot");
        }
        else if (g_ascii_strncasecmp (seq, "shutdown", 8) == 0)
        {
                background =
                        _splashy_theme_image_path (SPL_CURRENT_THEME,
                                        "/splashy/background/shutdown");
                if (!g_file_test (background, G_FILE_TEST_IS_REGULAR))
                        background =
                                _splashy_theme_image_path (SPL_DEFAULT_THEME,
                                                "/splashy/background/shutdown");
        }
        else if (g_ascii_strncasecmp (seq, "test", 4) == 0
                        || g_ascii_strncasecmp (seq, "preview", 7) == 0)
        {
                /*
                 * allows preview of splashy current theme and progressbar 
                 */
                _preview = TRUE;
                /*
                 * set bg image to boot 
                 */
                background =
                        _splashy_theme_image_path (SPL_CURRENT_THEME,
                                        "/splashy/background/boot");
                /*
                 * fallback to default theme if the current theme has no
                 * image defined 
                 */
                if (!g_file_test (background, G_FILE_TEST_IS_REGULAR))
                        background =
                                _splashy_theme_image_path (SPL_DEFAULT_THEME,
                                                "/splashy/background/boot");
        }
        else
        {
                background =
                        _splashy_theme_image_path (SPL_CURRENT_THEME,
                                        "/splashy/background/errorimg");
                if (!g_file_test (background, G_FILE_TEST_IS_REGULAR))
                        background =
                                _splashy_theme_image_path (SPL_DEFAULT_THEME,
                                                "/splashy/background/errorimg");
        }
        DEBUG_PRINT ("_splashy_child() starting splash with image <<%s>>",
                        background);
        if (!g_file_test (background, G_FILE_TEST_IS_REGULAR))
        {
                ERROR_PRINT ("fatal: background image not found <<%s>>",
                                background);
                splashy_child_exit ();
        }
        video_start_splash (&video, background);

        if (show_progress == TRUE)
        {
                /*
                 * starting bars stuff! 
                 */
                DEBUG_PRINT ("_splashy_child() starting progressbar %s", "");
                video_draw_progressbar (&video, &progressbar);
        }

        /*
         * setup our main input: keyboard 
         */
        /*
         * TODO would we leak if we don't check if video.keyboard is not NULL 
         * ? We should make sure splashy can't be lanched twice from this
         * same thread 
         */
        DFBCHECK (video.dfb->
                        GetInputDevice (video.dfb, DIDID_KEYBOARD,
                                &video.keyboard));
        /*
         * create event buffer for the keyboard: to listen for events 
         */
        DFBCHECK (video.keyboard->
                        CreateEventBuffer (video.keyboard, &video.ev_buffer));
        /*
         * here is where the magic happens: because WaitForEvent*() waits idle
         * for events, we need to fork and handle this in separate threads
         */

        /*
         * thread C 
         */
        thr_id_c =
                pthread_create (&p_thread_c, NULL, fifo_loop,
                                (void *) &_preview);

        /*
         * thread D 
         */
        thr_id_d = pthread_create (&p_thread_d, NULL, keyevent_loop, NULL);

        /*
         * while(1): wait till thread is finished reading the fifo... ie
         * 'forever' 
         */
        pthread_join (p_thread_c, NULL);

        return 0;               /* never reached */
}                               /* end _splashy_child() */

/**
 * @desc the preferred way to exit splashy:
 */
        void
splashy_child_exit ()
{
        cmd_exit (NULL);
}

/**
 * @desc what to do when splashy is starting up (boot process)
 * TODO
 * splashy child start/stop functions should allow argumenst from main.c
 */
        void
splashy_child_start ()
{
        DEBUG_PRINT ("Entering splashy_child_start %s", "");
        const gchar *show_progress =
                xml_parser_get_text ("/splashy/progressbaronboot");
        gboolean progress =
                (g_ascii_strncasecmp (show_progress, "yes", 3) ==
                 0) ? TRUE : FALSE;
        gint ret = _splashy_child ("boot", progress);
        if (ret < 0)
                ERROR_PRINT ("Could not start splashy. %d\n", ret);
        DEBUG_PRINT ("Exiting splashy_child_start %d", ret);
}

/**
 * @desc what to do when splashy is in the stopping process (shutdown)
 */
        void
splashy_child_stop ()
{
        const gchar *show_progress =
                xml_parser_get_text ("/splashy/progressbaronshutdown");
        gboolean progress =
                (g_ascii_strncasecmp (show_progress, "yes", 3) ==
                 0) ? TRUE : FALSE;
        gint ret = _splashy_child ("shutdown", progress);
        if (ret < 0)
                g_print ("ERROR: main process exited with %d\n", ret);
}

/**
 * @desc testing splashy
 */
        void
splashy_child_test ()
{
        DEBUG_PRINT ("Entering splashy_child_test %s", "");
        const gchar *show_progress =
                xml_parser_get_text ("/splashy/progressbaronboot");
        gboolean progress =
                (g_ascii_strncasecmp (show_progress, "yes", 3) ==
                 0) ? TRUE : FALSE;
        gint ret = _splashy_child ("test", progress);
        if (ret < 0)
                ERROR_PRINT ("Could not start splashy on test mode. %d\n",
                                ret);
        DEBUG_PRINT ("Exiting splashy_child_test %d", ret);
}
