/************************************************************************
 *                              main.c                                  *
 *                                                                      *
 *  2005-05-28 03:01 EDT                                                *
 *  Copyright  2005   Luis Mondesi    <lemsx1@gmail.com>                *
 *                                                                      *
 ************************************************************************/
/*
 * This file is part of Splashy.
 * 
 * Splashy is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation;
 * either version 2 of the License, or (at your option) any
 * later version.
 * 
 * Splashy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with Splashy; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA 02110-1301 USA 
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdlib.h>
#include <unistd.h>             /* fork() and related */
#include <glib.h>
#include <glib/gstdio.h>

#include "common_macros.h"
#include "functions.h"
#include "xml_parser.h"
#include "xml_config.h"

#define USAGE "usage: splashy {boot|shutdown|test}"

        int
main (int argc, char *argv[])
{
        DEBUG_PRINT ("main() invoked %d", argc);

        spl_test_env_pattern ("vga=", "/proc/cmdline");
        spl_test_env_pattern ("VESA|VGA", "/proc/fb");

        if (g_getenv ("RUNLEVEL") &&
                        g_ascii_strncasecmp (g_getenv ("RUNLEVEL"), "1", 1) == 0)
        {
                fprintf (stderr, "%s\n",
                                "Single user mode detected. Exiting ...");
                return 1;
        }

        if (argc < 2)
        {
                g_print ("%s\n", USAGE);
                return 1;
        }

        if (!spl_init ())       /* create FIFO or bail out */
        {
                fprintf (stderr, "%s\n",
                                "Error occured while starting splashy."
                                "Make sure that you have write access to the directory "
                                "holding splashy.fifo");
                /*
                 * TODO g_free() memory 
                 */
                return 1;
        }
        DEBUG_PRINT ("before child my pid is %d\n", getpid ());
        gint child = fork ();
        if (child < 0)
        {
                fprintf (stderr, "%s\n",
                                "Sorry, could not send splashy to the background. Bailing out...");
                exit (1);
        }
        if (child == 0)
        {
                /*
                 * child 
                 */
                DEBUG_PRINT ("child my parent pid is %d\n", getppid ());
                /*
                 * save your pid if file doesn't exist 
                 */
                const gchar *_pid_file = xml_parser_get_text ("/splashy/pid");
                /*
                 * We assume that there is NO splashy running. In fact, this shouldn't be our problem if it is already running. the pid file holds the PID of the last instance of splashy started. 
                 */
                FILE *file = g_fopen (_pid_file, "w+");
                if (file != NULL)
                {
                        fprintf (file, "%d\n", getpid ());
                        fclose (file);
                }
                /*
                 * handle arguments 
                 */
                if (g_ascii_strncasecmp (argv[1], "boot", 4) == 0)
                {
                        DEBUG_PRINT ("calling splashy_child_start(%s)", "");
                        splashy_child_start ();
                }
                else if (g_ascii_strncasecmp (argv[1], "shutdown", 8) == 0)
                {
                        DEBUG_PRINT ("calling splashy_child_stop(%s)", "");
                        splashy_child_stop ();
                }
                else if (g_ascii_strncasecmp (argv[1], "preview", 7) == 0
                                || g_ascii_strncasecmp (argv[1], "test", 4) == 0)
                {
                        DEBUG_PRINT ("calling splashy_child_test(%s)", "");
                        splashy_child_test ();
                }
                else
                {
                        DEBUG_PRINT ("No argument given %s", "");
                        g_print ("%s\n", USAGE);
                }
                splashy_child_exit ();
        }
        /*
         * parent 
         */
        DEBUG_PRINT ("after child my pid is %d\n", getpid ());
        DEBUG_PRINT ("after child my child pid is %d\n", child);
        return 0;
}
