/***************************************************************************
 *            splashy_config-functions.c
 *
 *  Thu Aug 25 16:11:09 2005
 *  Copyright  2005  Vincent Amouret
 *  vincent.amouret@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

#include <stdlib.h>     /* system,  exit */
#include <string.h>     /* strlen */
#include <unistd.h>     /* getuid */

#include "common_macros.h"
#include "xml_parser.h"
#include "xml_config.h"
#include "splashy_config-functions.h"

/* GLib */
#include <glib/gstdio.h>
#include <glib/gi18n.h>
/*
** Private function
*/
void
need_root								(void);

/**
 * @desc takes care of initilizing all special files needed
 * by splashy_config
 */
        gboolean
spl_config_init()
{
        DEBUG_PRINT("Entering spl_config_init(%s)","");
        if ( init_config(SPL_CONFIG_FILE) )
                return TRUE;
        return FALSE; /* assumes we will fail */
}

/*
** set_new_theme
** arg: The name of the theme to apply
** desc: Change the theme used by Splashy
*/
void
set_new_theme							(gchar* theme_name)
{
	need_root();
	GString* new_theme = g_string_new(xml_parser_get_text(SPL_THEMES_DIR));
	g_string_append_c(new_theme, '/');
	g_string_append(new_theme, theme_name);
	
    g_print(_(">Set theme as : "));
	g_print(theme_name);
	/*
	** Check if the theme exist
	*/
	if(g_file_test(new_theme->str, G_FILE_TEST_IS_DIR ))
	{		
		gen_config_xml((gchar*)xml_parser_get_text(SPL_THEMES_DIR), 
						theme_name, 
						(gchar*)xml_parser_get_text(SPL_FIFO_XPATH), 
						(gchar*)xml_parser_get_text("/splashy/pid"));		
		
		PRINT_DONE; 		
	}
	else
	{
		PRINT_FAIL;
	}
	
	g_string_free(new_theme, TRUE);
}

/*
** install_theme
** desc: install a theme from a tarball 
** arg: the tarball path 
*/
void
install_theme(gchar *tarball_path)
{
	need_root();	
	
	g_print(_(">Install theme"));
	
	gchar *pwd ;	
	GString* command = g_string_new("tar -xzf ");
	
	pwd = g_get_current_dir();	
	
	GString* target = g_string_new(xml_parser_get_text(SPL_THEMES_DIR));
	g_string_append_c(target, '/');
	g_string_append(target, tarball_path);
	g_string_insert_c(target, target->len-7, '\0');
		
	
	
	if(!g_path_is_absolute(tarball_path))
	{
		/*
		** Check if a theme with the same name exists
		** TODO - the same thing for an absolute path
		*/
		if(g_file_test(target->str, G_FILE_TEST_IS_DIR))
		{
			PRINT_FAIL;
			g_print(_("This theme already exists\n"));
			return;
		}
		
		g_string_append(command, pwd);
		g_string_append(command, "/");
		
	}
	g_string_append(command, tarball_path);	
	
	/*
	** Check if the tarball exists
	*/
	if(g_file_test(tarball_path, G_FILE_TEST_EXISTS))
	{
		chdir(xml_parser_get_text(SPL_THEMES_DIR));	
		if(system(command->str) != 0) 
		{
			PRINT_FAIL;
				g_print(_("An error occurred installing theme\n"));
		} else {
			PRINT_DONE;
		}
	
	} else {
		PRINT_FAIL;
		g_print(_("The tarball doesn't exist !\n"));
	}	
	
	g_free(pwd);
	g_string_free(command, TRUE);
}

/* 
** remove_theme
** desc: Remove a theme deleting his directory
** arg: the theme name
*/
void
remove_theme(gchar* theme_name)
{
	need_root();
	g_print(_(">Remove theme"));
	GString* path = g_string_new(xml_parser_get_text(SPL_THEMES_DIR));
	g_string_append_c(path, '/');
	g_string_append(path, theme_name);
	g_print(" %s", theme_name);
	
	
	const gchar* file = NULL;
	GString* buf = g_string_new("");
	if(!g_file_test(path->str, G_FILE_TEST_IS_DIR))
	{
		PRINT_FAIL;
		g_print(_("The theme doesn't exist !\n"));
	}
	else if( g_strcasecmp(theme_name, xml_parser_get_text(SPL_CURRENT_THEME)) == 0)
		{
			PRINT_FAIL;
			g_print(_("This theme is currently used !\n"));
			
		}
		else
		{
			DEBUG_PRINT("splashy_config: Reading files from %s \n", path->str);
			GDir* _dir = g_dir_open(path->str, 0, NULL);
			
			while( (file = g_dir_read_name(_dir)) )
			{
				
				g_string_assign(buf, path->str);
				g_string_append_c(buf, '/');
				g_string_append(buf, file);
				DEBUG_PRINT("splashy_config: Removing file %s \n", buf->str);
				
				if(remove(buf->str) != 0)
				{					
					PRINT_FAIL;
				}
				
			}
			g_dir_close(_dir);
			if(rmdir(path->str) !=0)
			{
				PRINT_FAIL;
			}
			else 
			{
				PRINT_DONE;					
			}
		}
	
	
	g_string_free(buf, TRUE);
	g_string_free(path, TRUE);
				
}

/*
** information
** desc: Display some informations
*/
void
information(void)
{
	g_print(_(">Theme currently used:\n"));
	g_print(  "                      %s\n", xml_parser_get_text("/splashy/current_theme"));
	g_print(  "                         %s\n",xml_parser_get_text("/splashy/theme/description"));
	g_print(_("                      by %s\n"),xml_parser_get_text("/splashy/theme/author"));
	
	g_print(_(">Installed themes:\n"));
        const gchar* _theme = NULL;
        const gchar* themes_dir = xml_parser_get_text(SPL_THEMES_DIR);
        chdir(themes_dir); /* TODO g_chdir belongs only to glib > 2.6 */
        DEBUG_PRINT("splashy_config: Reading themes from %s \n",themes_dir);
        GDir* _dir = g_dir_open(themes_dir,0,NULL); /* TODO use GError */
        while( (_theme = g_dir_read_name(_dir)) )
        {
                gchar* _theme_config = g_strconcat(_theme,"/config.xml",NULL);
                DEBUG_PRINT("splashy_config: testing theme dir %s\n",_theme_config);
                if ( g_file_test(_theme_config,G_FILE_TEST_IS_REGULAR) )
                {
                        g_print("                      %s\n", g_path_get_basename(_theme));
                }
        }
		g_dir_close(_dir);
}

/*
** A simple test if root priviledges are required
*/
void
need_root								(void)
{
	if(getuid() != 0)
	{
		g_print(_("<!> Insufficient priviledges <!>\nRoot priviledges require for this function\n"));
		exit(0);
	}
	
}

/*
** create_theme
** desc: create a new theme in an interactive mode
*/
void
create_theme							(XmlFields NewTheme)
{
	
	GString* bootimg;
	GString* shutdownimg;
	GString* errorimg;
	
	
	GString* path = g_string_new(xml_parser_get_text(SPL_THEMES_DIR));
	g_string_append_c(path, '/');
	g_string_append(path, NewTheme.name);
	
			
	if(g_mkdir(path->str, 0755)!=0)
	{
		
		g_print(_("Error ! Can't to create the directory\n"));		
		exit(-1);
	}
	
	/* upload the boot image */
	bootimg = g_string_new(path->str);
	g_string_append(bootimg, "/background.jpg");
	if( !upload_image(NewTheme.boot, bootimg->str))
	{
		
		g_print(_("Error ! Unable to upload the picture.\nAborting\n"));
		remove_theme(NewTheme.name);
		return ;
	}
	g_stpcpy(NewTheme.boot , bootimg->str);
	
	/* upload the shutdown image */
	shutdownimg = g_string_new(path->str);
	g_string_append(shutdownimg, "/shutdown.jpg");
	if( !upload_image(NewTheme.shutdown, shutdownimg->str))
	{
		
		g_print(_("Error ! Unable to upload the picture.\nAborting\n"));
		remove_theme(NewTheme.name);
		return ;
	}
	g_stpcpy(NewTheme.shutdown , shutdownimg->str);
	
	/* upload errorimg */
	errorimg = g_string_new(path->str);
	g_string_append(errorimg, "/error.jpg");
	if( !upload_image(NewTheme.error, errorimg->str))
	{
		
		g_print(_("Error ! Unable to upload the picture.\nAborting\n"));
		remove_theme(NewTheme.name);
		return ;
	}
	g_stpcpy(NewTheme.error , errorimg->str);
	
	
	g_string_append(path, "/config.xml");


	
	g_print(NewTheme.name);
	g_print(_(">Create theme "));
	if(gen_theme_xml(path->str, NewTheme)==0)
		PRINT_DONE;
	else
		PRINT_FAIL;
	g_string_free(path, TRUE);
	g_string_free(bootimg, TRUE);
	g_string_free(shutdownimg, TRUE);
	g_string_free(errorimg, TRUE);
}

/*
** get_fields 
** desc: The interactive mode for create_theme
*/
XmlFields 
get_fields(void)
{
	XmlFields NewTheme;
	
	g_print(_("--    Theme    --\n"));
	
	g_print(_(">Name       : ")) ;		
	get_string(NewTheme.name); 
	
	
	g_print(_(">Description: "));
	get_string(NewTheme.description);
	
	g_print(_(">Author     : "));
	get_string(	NewTheme.author);

	g_print(_("-- ProgressBar --\n"));
	g_print(_("**  Position   **\n"));	
	g_print(  "**   0->100    **\n");
	
	g_print(_(">X          : "));
	scanf("%u", &NewTheme.x);
	
	g_print(_(">Y          : "));
	scanf("%u", &NewTheme.y);
	
	g_print(_(">Width      : "));
	scanf("%u", &NewTheme.width);
	
	g_print(_(">Height     : "));
	scanf("%u", &NewTheme.height);
	
	g_print(_("**    Color    **\n"));
	g_print("**    0->255   **\n");

	g_print(_(">Red        : "));
	scanf("%u", &NewTheme.red);
	
	g_print(_(">Green      : "));
	scanf("%u", &NewTheme.green);
	
	g_print(_(">Blue       : "));
	scanf("%u", &NewTheme.blue);
	
	g_print(_(">Alpha      : "));
	scanf("%u", &NewTheme.alpha);
	
	
	g_print(_("-- Background --\n"));
	g_print(_("** Path to    **\n"));
	g_print(_("**upload files**\n"));
	g_print(_(">Boot       : "));
	scanf("%s", NewTheme.boot);
	
	g_print(_(">Shutdown   : ")) ;
	scanf("%s", NewTheme.shutdown);
	
	
	g_print(_(">Error      : "));
	scanf("%s", NewTheme.error);
	
	
	g_print(_(">Autoverbose"));
	g_print(" [yes | no]: ") ;
	scanf("%s", NewTheme.verbose);
	
	g_print(_(">Progressbar on boot"));
	g_print("[yes | no]: ");
	scanf("%s", NewTheme.progressbaronboot);
	
	g_print(_(">Progressbar on shutdown"));
	g_print("[yes | no]: ");
	scanf("%s", NewTheme.progressbaronshutdown);
	
	if(g_strcasecmp(NewTheme.verbose, "yes") != 0 && g_strcasecmp(NewTheme.verbose, "no") != 0)
		g_stpcpy(NewTheme.verbose, "yes");
		
	if(g_strcasecmp(NewTheme.progressbaronboot, "yes") != 0 && g_strcasecmp(NewTheme.progressbaronboot, "no") != 0)
		g_stpcpy(NewTheme.progressbaronboot, "yes");
	
	if(g_strcasecmp(NewTheme.progressbaronshutdown, "yes") != 0 && g_strcasecmp(NewTheme.progressbaronshutdown, "no") != 0)
		g_stpcpy(NewTheme.progressbaronshutdown, "yes");
	
	return NewTheme;
}

/*
** upload_image
** desc: upload a file
** args: src_path the source file
**       dest_path the destination file
*/
gboolean
upload_image(gchar* src_path, gchar* dest_path)
{
	FILE* src_file;
	FILE* dest_file;
	struct stat file;
	gulong filesize = 0;
	gchar* content;	
	
	g_print(_("Upload file : "));
	g_print(src_path);
	if((src_file = fopen(src_path, "r")) == NULL)
	{
		
		PRINT_FAIL;
		//g_critical("Error opening file %s\n", src_path);
		return FALSE;
	}
	
     
     if(!g_stat(src_path,&file))
     {
         filesize  = file.st_size;
     }
	 content = (gchar*)g_malloc(filesize);
	 fread(content, 1 , filesize, src_file);
	 fclose(src_file);
	 
	
	if((dest_file = fopen(dest_path, "w+")) == NULL)
	{
		
		PRINT_FAIL;
		g_critical("Error opening dest file %s\n", dest_path);
		return FALSE;
	}
	fwrite(content, 1, filesize, dest_file);
	fclose(dest_file);
	
	g_free(content);	
	PRINT_DONE;
	return TRUE;
	
}
