/***************************************************************************
 *            splashy_config-main.c
 *
 *  Sun Sep 11 17:37:34 2005
 *  Copyright  2005  Vincent Amouret
 *  vincent.amouret@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */
 
#include <unistd.h> /* getuid */
#include <getopt.h> /* getopt_long */
#include <glib/gi18n.h> 

#include "splashy_config-functions.h"

#define PROGNAME "splashy_config"
#define HELP \
_("------------------\n\
SPLASHY-CONFIG 0.1\n\
------------------\n\
A Splashy Configuration Tool\n\
\n\
usage: %s [option] arg \n\
\n\
-h, --help                          Display this help\n\
-s, --set-theme THEME               Set THEME as the splashy theme\n\
-i, --install-theme THEME.tar.gz    Install the THEME from a tarball\n\
-r, --remove-theme THEME            Remove the THEME\n\
-u, --upload-image  SOURCE DEST     Upload the image SOURCE to the DEST file path\n\
    --info                          Display some informations\n\
-c, --create-theme  [args...]       Create a theme in an interactive mode if no arguments\n\
                                    Inline mode arguments (this mode must be used with precautions)\n\
                    -name  NAME          Set the NAME of the new splashy theme\n\
                    -description DESC    Set the DESC of the new splashy theme\n\
                    -author AUTHOR       Set the AUTHOR(s) of the new splashy theme\n\
                    -x X-POSITION        Set the X-POSITION of the progress bar (%%)\n\
                    -y Y-POSITION        Set the Y-POSITION of the progress bar (%%)\n\
                    -width WIDTH         Set the WIDTH of the progress bar      (%%)\n\
                    -height HEIGHT       Set the height of the progress bar     (%%)\n\
                    -red RED             Set the RED channel of the progress bar   (0->255)\n\
                    -green GREEN         Set the GREEN channel of the progress bar (0->255)\n\
                    -blue BLUE           Set the BLUE channel of the progress bar  (0->255)\n\
                    -alpha ALPHA         Set the ALPHA channel of the progress bar (0->255)\n\
                    -boot-image PATH     Upload the image located at the specified PATH\n\
                    -shutdown-image PATH Upload the image located at the specified PATH\n\
                    -error-image PATH    Upload the image located at the specified PATH\n\
                    -autoverbose yes|no  Set if splashy turn into verbose mode when an error appears at boot\n\
		    -progressbaronboot yes|no\n\
		    -progressbaronshutdown yes|no \n")
#define USAGE \
_("usage: %s [option] arg \n\
[-h, --help][-s, --set-theme THEME][-i, --install-theme THEME.tar.gz]\n\
[-r, --remove-theme THEME][-u, --upload-image  SOURCE DEST][--info]\n\
[-c, --create-theme  [args...]]\n\
Show the help for more informations\n")

static struct option long_options[] = {
									{ "set-theme"    , required_argument, 0, 's' },
									{ "install-theme", required_argument, 0, 'i' },
									{ "remove-theme" , required_argument, 0, 'r' },
									{ "info"         , no_argument      , 0, 'a' },
									{ "create-theme" , optional_argument, 0, 'c' },
									{ "upload-image" , required_argument, 0, 'u' },
									{ "help"         , no_argument      , 0, 'h' },
									{0,0,0,0}
								};
static struct option create_options[] = {
									{ "name"             , required_argument, 0, 'n' },
									{ "description"      , required_argument, 0, 'd' },
									{ "author"           , required_argument, 0, 'a' },
									{ "x"                , required_argument, 0, 'x' },
									{ "y"                , required_argument, 0, 'y' },
									{ "width"            , required_argument, 0, 'w' },
									{ "height"           , required_argument, 0, 'h' },
									{ "red"              , required_argument, 0, 'r' },
									{ "green"            , required_argument, 0, 'g' },
									{ "blue"             , required_argument, 0, 'b' },
									{ "alpha"            , required_argument, 0, 'A' },
									{ "boot-image"       , required_argument, 0, 'B' },
									{ "shutdown-image"   , required_argument, 0, 's' },
									{ "error-image"      , required_argument, 0, 'e' },
									{ "autoverbose"		 , required_argument, 0, 'v' },
									{ "progressbaronboot", required_argument, 0, 'p' },
									{ "progressbaronshutdown", required_argument, 0, 'P' },
									{0,0,0,0}									
								};	
gint main(gint argc, gchar *argv[])
{
        if ( ! spl_config_init() ) /* create FIFO or bail out */
        {
                fprintf(stderr, "%s\n","Error occured while starting splashy."
                                "Make sure that you have write access to the directory "
                                "holding splashy.fifo");
                /* g_free() memory */
                return 1;
        }
	g_set_prgname(PROGNAME);
	
	XmlFields inline_theme;
	gint retopt = 0;
	gint create_retopt = 0;
	gint option_index = 0;
	gint create_option_index = 0;
	retopt = getopt_long(argc, argv, "hs:i:r:ic::u:", long_options, &option_index);
	if(retopt == -1)
	{
		printf("Missing arguments\n");
		printf(USAGE, g_get_prgname());
		return 0;
	}
	switch(retopt)
	{
		case 's':
			set_new_theme(optarg);
			break;
		case 'i':
			install_theme(optarg);
			break;
		case 'r':
			remove_theme(optarg);
			break;
		case 'a':
			information();
			break;
		case 'c':
			if(argv[optind] == NULL)// interactive mode
				create_theme(get_fields());
			else// inline mode
			{
				while(argc>optind)
				{
					create_retopt = getopt_long_only(argc, argv, "", create_options, &create_option_index);					
					switch(create_retopt)
					{
						case 'n':
							g_stpcpy(inline_theme.name, optarg);
							break;
						case 'd':
							g_stpcpy(inline_theme.description , optarg);
							break;
						case 'a':
							g_stpcpy(inline_theme.author, optarg);
							break;
						case 'x':
							inline_theme.x = g_strtod(optarg, 0);
							break;
						case 'y':
							inline_theme.y = g_strtod(optarg, 0);
							break;
						case 'w':
							inline_theme.width = g_strtod(optarg, 0);
							break;
						case 'h':
							inline_theme.height = g_strtod(optarg, 0);
							break;
						case 'r':
							inline_theme.red = g_strtod(optarg, 0);
							break;
						case 'g':
							inline_theme.green = g_strtod(optarg, 0);
							break;
						case 'b':
							inline_theme.blue = g_strtod(optarg, 0);
							break;
						case 'A':
							inline_theme.alpha = g_strtod(optarg, 0);
							break;
						case 'B':
							g_stpcpy(inline_theme.boot, optarg);
							break;
						case 's':
							g_stpcpy(inline_theme.shutdown, optarg);
							break;
						case 'e':
							g_stpcpy(inline_theme.error, optarg);
							break;
						case 'v':
							g_stpcpy(inline_theme.verbose, optarg);
							break;
						case 'p':
							g_stpcpy(inline_theme.progressbaronboot, optarg);
							break;
						case 'P':
							g_stpcpy(inline_theme.progressbaronshutdown, optarg);
							break;
					}
				}
				create_theme(inline_theme);
			}
			break;
		case 'u':
			upload_image(optarg, argv[optind]);
			break;		
		case 'h':
			printf(HELP, g_get_prgname());
			break;
		case '?':
			printf(USAGE, g_get_prgname());
			break;
	}
	
	
	return (0);	
}
