/************************************************************************
 *                              video.c                                 *
 *                                                                      *
 *  2005-05-28 03:01 EDT                                                *
 *  Copyright  2005  Otavio Salvador <otavio@debian.org>                *
 *                   Luis Mondesi    <lemsx1@gmail.com>                 *
 ************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdlib.h>             /* abs() */
#include <directfb.h>
#include <stdio.h>
#include <string.h>
#include <time.h>               /* nanosleep() */
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <linux/fb.h>
#include "common_macros.h"
#include "xml_parser.h"
#include "xml_config.h"
#include "video.h"

static gint _last_progress = 0, _last_w = 0;
static const gchar *_current_background = NULL;
static gchar *fb_dev_name = NULL;       /* such as /dev/fb0 */
static gint fb_dev_fd;          /* handle for fb_dev_name */
static struct fb_var_screeninfo fb_vinfo;       /* variable info */
static struct fb_var_screeninfo fb_orig_vinfo;  /* variable info to restore
                                                 * later */
static gint fb_bpp;             /* 32: 32 24: 24 16: 16 15: 15 */

/*FIXME do we need this here? */
static IDirectFBFont *font = NULL;
/* end FIXME */

/*
 * helper functions 
 */
        static gint
fb_preinit (gint reset)
{
        static gint fb_preinit_done = 0;
        static gint fb_err = -1;

        if (reset)
        {
                fb_preinit_done = 0;
                return 0;
        }

        if (fb_preinit_done)
                return fb_err;
        fb_preinit_done = 1;

        if (!fb_dev_name && !(fb_dev_name = getenv ("FRAMEBUFFER")))
                fb_dev_name = g_strdup ("/dev/fb0");

        DEBUG_PRINT ("Using device %s\n", fb_dev_name);

        if ((fb_dev_fd = open (fb_dev_name, O_RDWR)) == -1)
        {
                fprintf (stderr, "ERROR: Can't open %s: %s\n", fb_dev_name,
                                strerror (errno));
                goto err_out;
        }
        if (ioctl (fb_dev_fd, FBIOGET_VSCREENINFO, &fb_vinfo))
        {
                ERROR_PRINT ("[fbdev2] Can't get VSCREENINFO: %s\n",
                                strerror (errno));
                goto err_out;
        }
        fb_orig_vinfo = fb_vinfo;

        fb_bpp = fb_vinfo.bits_per_pixel;

        /*
         * 16 and 15 bpp is reported as 16 bpp 
         */
        if (fb_bpp == 16)
                fb_bpp = fb_vinfo.red.length + fb_vinfo.green.length +
                        fb_vinfo.blue.length;

        fb_err = 0;
        return 0;
err_out:
        if (fb_dev_fd >= 0)
                close (fb_dev_fd);
        fb_dev_fd = -1;
        fb_err = -1;
        return -1;
}

        static guint
preinit (const char *subdevice)
{
        if (subdevice)
        {
                if (fb_dev_name)
                        free (fb_dev_name);
                fb_dev_name = strdup (subdevice);
        }
        return fb_preinit (0);
}

/*
 * Public members 
 */
        void
video_allow_vt_switching ()
{
        DEBUG_PRINT ("video_allow_vt_switching(%s) called", "");
        /*
         * Please read functions.c about why NOT to set vt-switching here.
         * 2005-04-28 20:42 EDT - Luis Mondesi <lemsx1@gmail.com> 
         */
        /*
         * DirectFBSetOption( "vt-switching", NULL); 
         */
        DirectFBSetOption ("force-windowed", NULL);
}

        gint
video_draw_progressbar (splashy_video_t * video, DFBRectangle * progressbar)
{
        gint screen_width, screen_height;
        guint retangle_red, retangle_green, retangle_blue, retangle_alpha;
        
        const gchar * draw_progress = xml_parser_get_text ("/splashy/progressbar/border");
        gboolean draw_progress_border = ( draw_progress && g_ascii_strncasecmp(draw_progress,"yes",3) == 0 ) ? TRUE : FALSE;

        DEBUG_PRINT("Printing progress border: %s",draw_progress_border);
        video->surface->GetSize (video->surface, &screen_width, &screen_height);

        progressbar->x = screen_width *
                xml_parser_get_int ("/splashy/progressbar/x", 10) / 100;
        progressbar->y = screen_height *
                xml_parser_get_int ("/splashy/progressbar/y", 10) / 100;
        progressbar->w = screen_width *
                xml_parser_get_int ("/splashy/progressbar/width", 10) / 100;
        progressbar->h = screen_height *
                xml_parser_get_int ("/splashy/progressbar/height", 10) / 100;

        if (progressbar->x < 0 ||
                        progressbar->y < 0 || progressbar->w < 0 || progressbar->h < 0)
        {
                g_print ("ERROR: progress bar coordinates (x,y,width,height) had negative values. Please check your configuration exists and it's readable\n");
                return 1;
        }

        if ( draw_progress_border == TRUE )
        {
                DEBUG_PRINT ("draw x = %d", progressbar->x);
                DEBUG_PRINT ("draw y = %d", progressbar->y);
                DEBUG_PRINT ("draw w = %d", progressbar->w);
                DEBUG_PRINT ("draw h = %d", progressbar->h);

                video->surface->DrawRectangle (video->surface,
                                progressbar->x,
                                progressbar->y,
                                progressbar->w, progressbar->h);
        }
        retangle_red = xml_parser_get_int ("/splashy/progressbar/red", 10);
        retangle_green =
                xml_parser_get_int ("/splashy/progressbar/green", 10);
        retangle_blue = xml_parser_get_int ("/splashy/progressbar/blue", 10);
        retangle_alpha =
                xml_parser_get_int ("/splashy/progressbar/alpha", 10);

        /*
         * set the filling color of the progressbar to be used by video_update_progress()
         */
        if (retangle_red > 255 ||
                        retangle_green > 255 ||
                        retangle_blue > 255 || retangle_alpha > 255)
        {
                g_print ("ERROR: red, green, blue or alpha tags in the configuration file contain wrong values");
                return 3;
        }
        DEBUG_PRINT ("draw red   = %d", retangle_red);
        DEBUG_PRINT ("draw green = %d", retangle_green);
        DEBUG_PRINT ("draw blue  = %d", retangle_blue);
        DEBUG_PRINT ("draw alpha = %d", retangle_alpha);
        video->surface->SetColor (video->surface,
                        retangle_red,
                        retangle_green,
                        retangle_blue, retangle_alpha);
        return 0;
}

        gint
video_update_progressbar (splashy_video_t * video,
                DFBRectangle * progressbar, gint perc)
{
        DEBUG_PRINT ("video_update_progressbar(%d) called", perc);
        if (perc < 0)
                return 1;       /* sanity check */
        if (video == NULL)
                return 1;
        DEBUG_PRINT ("video_update_progressbar(%d) sanity checks passed",
                        perc);
        DEBUG_PRINT ("progress x = %d", progressbar->x);
        DEBUG_PRINT ("progress y = %d", progressbar->y);
        DEBUG_PRINT ("progress w = %d", progressbar->w);
        DEBUG_PRINT ("progress h = %d", progressbar->h);
        /*
         * how many nanoseconds we sleep while updating the progress bar 
         */
        struct timespec _test_sleep;
        _test_sleep.tv_sec = 0;
        _test_sleep.tv_nsec = 100;

        if (perc > 100 || _last_progress >= 100)
        {
                /*
                 * TODO this is too slow... find a better way to redraw
                 * progress 
                 *
                 * we need to re-draw the progressbar when it's full 
                 */
                video->surface->SetColor (video->surface, 0, 0, 0, 0);
                video_change_splash (video, _current_background);
                video_draw_progressbar (video, progressbar);
                _last_w = 0;
        }

        guint i = 0;
        guint _w = progressbar->w * perc / 100;
        DEBUG_PRINT ("Progress width %d", _w);

        /*
         * draw (smooth) progress forward 
         */
        for (i = _last_w; i <= _w; i++)
        {
                DEBUG_PRINT ("Updating progress by %d", i);
                video->surface->FillRectangle (video->surface,
                                progressbar->x,
                                progressbar->y,
                                i, progressbar->h);
                nanosleep (&_test_sleep, NULL);
        }
        _last_w = _w;
        _last_progress = perc;
        return 0;
}

        static DFBEnumerationResult
video_modes_callback (gint width, gint height, gint bpp, void *data)
{
        spl_videomode_t *video = (spl_videomode_t *) data;
        gint overx = 0, overy = 0, closer = 0, over = 0;
        gint we_are_under = 0;

#ifdef DEBUG
        gchar *msg =
                g_strdup_printf
                ("video_modes_callback: Validator entered %i %i %i\n", width,
                 height, bpp);
        DEBUG_PRINT ("%s", msg);
        g_free (msg);
#endif
        overx = width - video->out_width;
        overy = height - video->out_height;
        if (!video->width)
        {
                video->width = width;
                video->height = height;
                video->overx = overx;
                video->overy = overy;
#ifdef DEBUG
                gchar *msg =
                        g_strdup_printf
                        ("video_modes_callback: Mode added %i %i %i\n", width,
                         height, bpp);
                DEBUG_PRINT ("%s", msg);
                g_free (msg);
#endif
        }
        if ((video->overy < 0) || (video->overx < 0))
                we_are_under = 1;       /* stored mode is smaller than req
                                         * mode */
        if (abs (overx * overy) < abs (video->overx * video->overy))
                closer = 1;     /* current mode is closer to desired res */
        if ((overx >= 0) && (overy >= 0))
                over = 1;       /* current mode is bigger or equaul to
                                 * desired res */
        if ((closer && (over || we_are_under)) || (we_are_under && over))
        {
                video->width = width;
                video->height = height;
                video->overx = overx;
                video->overy = overy;
#ifdef DEBUG
                gchar *msg =
                        g_strdup_printf
                        ("video_modes_callback: Better mode added %i %i %i\n",
                         width, height, bpp);
                DEBUG_PRINT ("%s", msg);
                g_free (msg);
#endif
        };
        return DFENUM_OK;
}

        void
video_set_mode (splashy_video_t * video)
{
        DFBResult ret =
                video->dfb->EnumVideoModes (video->dfb, video_modes_callback,
                                video->mode);
        if (ret)
        {
                DEBUG_PRINT
                        ("%s: Error while detecting Full Screen Video Modes",
                         "video->dfb->EnumVideoModes");
        }
        ret = video->dfb->SetVideoMode (video->dfb, video->mode->width,
                        video->mode->height,
                        video->mode->bpp);
        if (ret)
        {
                /*
                 * 24 bit works for almost all modern video cards 
                 */
                ret = video->dfb->SetVideoMode (video->dfb,
                                video->mode->width,
                                video->mode->height, 24);
                if (ret)
                {
                        /*
                         * what about 32 bits? 
                         */
                        ret = video->dfb->SetVideoMode (video->dfb,
                                        video->mode->width,
                                        video->mode->height,
                                        32);
                        if (ret)
                        {
                                /*
                                 * well, let's go down to 16 bit 
                                 */
                                ret = video->dfb->SetVideoMode (video->dfb,
                                                video->mode->
                                                width,
                                                video->mode->
                                                height, 16);
                                if (ret)
                                {
                                        /*
                                         * @#$@#$ 8 bit!? 
                                         */
                                        ret = video->dfb->
                                                SetVideoMode (video->dfb,
                                                                video->mode->
                                                                width,
                                                                video->mode->
                                                                height, 8);
                                        DEBUG_PRINT
                                                ("%s: warning using 8 bit mode",
                                                 "video->dfb->EnumVideoModes");
                                }
                                else
                                {
                                        DEBUG_PRINT
                                                ("%s: Error while detecting bpp. can't use 24, 32, 16 or 8 bits-per-pixel in this system",
                                                 "video->dfb->EnumVideoModes");
                                }
                        }
                        else
                        {
                                DEBUG_PRINT ("%s: warning using 32 bit mode",
                                                "video->dfb->EnumVideoModes");
                        }
                }
                else
                {
                        DEBUG_PRINT ("%s: warning using 24 bit mode",
                                        "video->dfb->EnumVideoModes");
                }
        }
        DEBUG_PRINT ("Set resolution for %s:", "video->dfb");
        DEBUG_PRINT ("Width: %d", video->mode->width);
        DEBUG_PRINT ("Height: %d", video->mode->height);
        /*
         * used to disable inputs from mouse and reduce overhead (about 15%
         * of CPU usage)
         */
        video->dfb->SetCooperativeLevel (video->dfb, DFSCL_FULLSCREEN);
}

        void
video_start_splash (splashy_video_t * video, const gchar * background)
{
        /*
         * initializing Directfb
         */
        /*
         * - no-debug           := suppresses debug messages
         * - quiet              := suppresses all messages (but debug)
         * - graphics-vt        := puts directfb vt in graphics mode
         * - no-cursor          := disallow showing a cursor
         */

        _current_background = background;

        DFBCHECK (DirectFBInit (NULL, NULL));
        DirectFBSetOption ("quiet", NULL);
        DirectFBSetOption ("no-debug", NULL);
        DirectFBSetOption ("graphics-vt", NULL);
        DirectFBSetOption ("no-cursor", NULL);
        /*
         * in window-mode our dfb seems to prefer to go to the top layer
         * which masks the progress bar! commented out (libdirectfb-0.9.22)
         * This also makes the progressbar draw slower!
         */
        //DirectFBSetOption( "force-windowed", NULL); 
         

        DFBCHECK (DirectFBCreate (&video->dfb));

        video->mode = g_new0 (spl_videomode_t, 1);
        /*
         * set our expectation to a very big number 
         */
        preinit (NULL);
        video->mode->out_height = fb_vinfo.yres;
        video->mode->out_width = fb_vinfo.xres;

        DEBUG_PRINT ("Setting min Width (x) resolution to %d",
                        video->mode->out_width);
        DEBUG_PRINT ("Setting min Height (y) resolution to %d",
                        video->mode->out_height);

        video_set_mode (video);

        DFBCHECK (video->dfb->CreateImageProvider (video->dfb, background,
                                &video->provider));
        DFBCHECK (video->provider->GetSurfaceDescription (video->provider,
                                &video->desc));
        /*
         * TODO use CreateInputEventBuffer with caps to allow the fullscreen
         * surface to grab keyboard events. this need to be store in the
         * video struct 
         */
        /*
         * flags to set the default surface as main surface
         */
        video->desc.flags = DSDESC_CAPS;
        video->desc.caps = DSCAPS_PRIMARY;

        DFBCHECK (video->dfb->CreateSurface (video->dfb, &video->desc,
                                &video->surface));

        /*FIXME DFBCHECK (video->dfb->LowerToBottom (video->dfb)); */
        /*
         * allow surface to have alpha channels 
         */
        DFBCHECK (video->surface->
                        SetBlittingFlags (video->surface,
                                DSBLIT_BLEND_ALPHACHANNEL));

        /*
         * it writes on the framebuffer the background image
         */
        video->provider->RenderTo (video->provider, video->surface, NULL);
}

        void
video_change_splash (splashy_video_t * video, const gchar * newimage)
{
        video->dfb->CreateImageProvider (video->dfb, newimage,
                        &video->provider);
        video->provider->RenderTo (video->provider, video->surface, NULL);
}

/**
 * FIXME work in progress ...
 * When pressing F2, we show the scrolling UNIX text, in a smaller window
 *
 * @return
 */
        void
video_verbose_text(splashy_video_t * video)
{
        gint i, width;
        gchar * text = "testing splashy text with foo bar";
        DFBFontDescription font_dsc;
        /* DFBSurfaceDescription dsc; */
        font_dsc.flags = DFDESC_HEIGHT;
        font_dsc.height = 48;

        gint screen_width, screen_height;
        video->surface->GetSize (video->surface, &screen_width, &screen_height);

        DFBCHECK (video->dfb->CreateFont (video->dfb, "/usr/share/fonts/truetype/freefont/FreeMono.ttf", &font_dsc, &font));
        DFBCHECK (video->surface->SetFont (video->surface, font));
        DFBCHECK (font->GetStringWidth (font, text, -1, &width));

        /* We want to let the text slide in on the right and slide out on the left. */
        for (i = screen_width; i > -width; i--)
        {
                DFBCHECK (video->surface->SetColor (video->surface, 0x0, 0x0, 0x0, 0xFF));
                /*  Clear the screen. */
                DFBCHECK (video->surface->FillRectangle (video->surface, 0, 0, screen_width, screen_height));
                /* Set the color that will be used to draw the text. */
                DFBCHECK (video->surface->SetColor (video->surface, 0x80, 0x0, 0x20, 0xFF));
                /* Draw the text left aligned with "i" as the X coordinate. */
                DFBCHECK (video->surface->DrawString (video->surface, text, -1, i, screen_height / 2, DSTF_LEFT));
                /* Flip the front and back buffer, but wait for the vertical retrace to avoid tearing. */
                DFBCHECK (video->surface->Flip (video->surface, NULL, DSFLIP_WAITFORSYNC));
        }
        font->Release (font);
}
